#include "PhaseFieldCrystal_Base.h"

using namespace std;
using namespace AMDiS;

PhaseFieldCrystal::PhaseFieldCrystal(const std::string &name_) :
  super(name_),
  useMobility(false),
  tempParameter(-0.6),
  r(-0.4),		// temperature deviation
  rho0(1.0),		// liquid density
  density(-0.3),	// mean density
  two(2.0),
  minus2(-2.0)
{  
  Parameters::get(name + "->r",r);
  Parameters::get(name + "->rho0", rho0);
  Parameters::get(name + "->density", density);
  Parameters::get(name + "->use mobility", useMobility);
  tempParameter= -(1.0+r);
};


void PhaseFieldCrystal::fillOperators()
{ FUNCNAME("PhaseFieldCrystal::fillOperators()");
  
  // phi*rho
  Operator *opMnew = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMnew->addTerm(new Simple_ZOT);
  Operator *opMold = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMold->addTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0), NULL));

  // -nabla*(phi*grad(rho))
  Operator *opL = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL->addTerm(new Simple_SOT);
  
  Operator *opLM = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  if (useMobility) // non-constant mobility
    opLM->addTerm(new VecAtQP_SOT(
      prob->getSolution()->getDOFVector(0),
      new MobilityPfc(density, rho0)));
  else // constant mobility
    opLM->addTerm(new Simple_SOT);
  
  // -(1+r)*phi*rho
  Operator *opMTemp = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMTemp->addZeroOrderTerm(new Simple_ZOT());
  // -2*rho_old^3
  Operator *opMPowExpl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowExpl->addZeroOrderTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new Pow3Functor(-2.0)));
  // -3*rho_old^2
  Operator *opMPowImpl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowImpl->addZeroOrderTerm(new VecAtQP_ZOT(prob->getSolution()->getDOFVector(0),new Pow2Functor(-3.0)));
  
  // 0 = nu-laplace(rho)
  // -------------------
  prob->addMatrixOperator(opL, 0, 0); // -laplace(rho)
  prob->addMatrixOperator(opMnew, 0, 2); // nu
  setAssembleMatrixOnlyOnce_butTimestepChange(0,0);
  setAssembleMatrixOnlyOnce_butTimestepChange(0,2);
  
  // dt(rho) = laplace(mu) - u*grad(rho)
  // -----------------------------------
  prob->addMatrixOperator(opMnew, 1, 0, getInvTau(), getInvTau());
  prob->addMatrixOperator(opLM, 1, 1); // -laplace(mu)
  // . . . vectorOperators . . . . . . . . . . . . . . .
  prob->addVectorOperator(opMold, 1, getInvTau(), getInvTau());
  setAssembleMatrixOnlyOnce_butTimestepChange(1,0);
  if (!useMobility)
    setAssembleMatrixOnlyOnce_butTimestepChange(1,1);
  
  // mu-2*nu-laplace(nu)-(1+r)*rho = (rho_old^3) + ExtPot - eps^2/(rho_old+0.9)
  // ----------------------------------------------------------------------
  prob->addMatrixOperator(opMTemp, 2, 0, getTempParameter(), getTempParameter()); // -phi*(1+r)*rho
  prob->addMatrixOperator(opMPowImpl, 2, 0); // -3*rho*rho_old^2
  prob->addMatrixOperator(opL, 2, 0, &two, &two); // -2*phi*laplace(rho) * psi
  prob->addMatrixOperator(opMnew, 2, 1); // phi*mu * psi
  prob->addMatrixOperator(opL, 2, 2); // phi*grad(nu) * grad(psi)
  // prob.addMatrixOperator(opMnew,2,2,&minus2);
  // . . . vectorOperators . . . . . . . . . . . . . . .
  prob->addVectorOperator(opMPowExpl, 2); // -2*phi^old*rho_old^3
  
  setAssembleMatrixOnlyOnce_butTimestepChange(2,1);
  setAssembleMatrixOnlyOnce_butTimestepChange(2,2);
};


void PhaseFieldCrystal::fillBoundaryConditions()
{ FUNCNAME("PhaseFieldCrystal::fillBoundaryConditions()");
};
