#include "PhaseFieldCrystal_Phase.h"

using namespace std;
using namespace AMDiS;

PhaseFieldCrystal_Phase::PhaseFieldCrystal_Phase(const std::string &name_) :
  PhaseFieldCrystal(name_),
  phase(NULL),
  phaseDisturbed(NULL)
{};


PhaseFieldCrystal_Phase::~PhaseFieldCrystal_Phase()
{
  delete phase;
  delete phaseDisturbed;
}


void PhaseFieldCrystal_Phase::initTimeInterface()
{ FUNCNAME("PhaseFieldCrystal_Phase::initTimeInterface()");

  PhaseFieldCrystal::initTimeInterface();
};


void PhaseFieldCrystal_Phase::fillOperators()
{ FUNCNAME("PhaseFieldCrystal_Phase::fillOperators()");
  MSG_DBG("PhaseFieldCrystal_Phase::fillOperators()\n");
  
  // phi*rho
  Operator *opMnew = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMnew->addTerm(new Phase_ZOT(phaseDisturbed));
  Operator *opMold = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMold->addTerm(new Phase_ZOT(phaseDisturbed));

  // -nabla*(phi*grad(rho))
  Operator *opL = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opL->addTerm(new Phase_SOT(phaseDisturbed));
  
  Operator *opLM = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  if (useMobility) // non-constant mobility
      opLM->addTerm(new Vec2AtQP_SOT(
        phaseDisturbed,
        prob->getSolution()->getDOFVector(0),
        new MobilityPhasePfc(density, rho0)));
  else // constant mobility
      opLM->addTerm(new Phase_SOT(phaseDisturbed));
  
  // -(1+r)*phi*rho
  Operator *opMTemp = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMTemp->addZeroOrderTerm(new Phase_ZOT(phaseDisturbed));
  // -2*rho_old^3
  Operator *opMPowExpl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowExpl->addZeroOrderTerm(new Pow3Phase_ZOT(phaseDisturbed,prob->getSolution()->getDOFVector(0),-2.0));
  // -3*rho_old^2
  Operator *opMPowImpl = new Operator(prob->getFeSpace(0), prob->getFeSpace(0));
  opMPowImpl->addZeroOrderTerm(new Pow2Phase_ZOT(phaseDisturbed,prob->getSolution()->getDOFVector(0),-3.0));
  
  // 0 = nu-laplace(rho)
  // -------------------
  prob->addMatrixOperator(opL, 0, 0); // -laplace(rho)
  prob->addMatrixOperator(opMnew, 0, 2); // nu

  // dt(rho) = laplace(mu) - u*grad(rho)
  // -----------------------------------
  prob->addMatrixOperator(opMnew, 1, 0, getInvTau());
  prob->addMatrixOperator(opLM, 1, 1); // -laplace(mu)
  // . . . vectorOperators . . . . . . . . . . . . . . .
  opMold->setUhOld(prob->getSolution()->getDOFVector(0));
  prob->addVectorOperator(opMold, 1, getInvTau());
  
  // mu-2*nu-laplace(nu)-(1+r)*rho = (rho_old^3) + ExtPot - eps^2/(rho_old+0.9)
  // ----------------------------------------------------------------------
  prob->addMatrixOperator(opMTemp, 2, 0, getTempParameter()); // -phi*(1+r)*rho
  prob->addMatrixOperator(opMPowImpl, 2, 0); // -3*rho*rho_old^2
  prob->addMatrixOperator(opL, 2, 0, &two); // -2*phi*laplace(rho) * psi
  prob->addMatrixOperator(opMnew, 2, 1); // phi*mu * psi
  prob->addMatrixOperator(opL, 2, 2); // phi*grad(nu) * grad(psi)
  // prob.addMatrixOperator(opMnew,2,2,&minus2);
  // . . . vectorOperators . . . . . . . . . . . . . . .
  prob->addVectorOperator(opMPowExpl, 2); // -2*phi^old*rho_old^3
};
